# -*- coding: utf-8 -*-

"""A getter for UMLS."""

from datetime import datetime

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "UMLSGetter",
]

URL = "https://www.nlm.nih.gov/research/umls/licensedcontent/umlsknowledgesources.html"


class UMLSGetter(Getter):
    """A getter for UMLS."""

    bioregistry_id = "umls"
    name = "UMLS"
    version_type = VersionType.other

    def get(self) -> datetime:
        """Get the latest UMLS version number."""
        soup = get_soup(URL)
        version_tag = (
            soup.find("main")
            .find("div", {"class": "grid-row grid-gap-1"})
            .find("div", {"class": "tablet:grid-col-12"})
            .find("h2")
        )
        version = version_tag.text.split()[0]
        return version


if __name__ == "__main__":
    UMLSGetter.print()



# -*- coding: utf-8 -*-

"""A getter for ICF."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "ICFGetter",
]

URL = "https://icd.who.int/browse/latest-release/icf/en"


class ICFGetter(Getter):
    """A getter for ICF."""

    bioregistry_id = "icf"
    name = "International Classification of Functioning, Disability and Health"
    version_type = VersionType.date
    date_version_fmt = "%Y-%m"

    def get(self) -> str:
        """Get the latest ICF version number."""
        response = requests.get(URL, allow_redirects=True)
        final_url = response.url
        return final_url[len("https://icd.who.int/browse/") :].split("/")[0]


if __name__ == "__main__":
    ICFGetter.print()



# -*- coding: utf-8 -*-

"""A getter for RxNorm."""

from datetime import datetime

from ..utils import Getter, VersionType, get_soup

__all__ = [
    "RxNormGetter",
]

URL = "https://www.nlm.nih.gov/research/umls/rxnorm/docs/rxnormfiles.html"


class RxNormGetter(Getter):
    """A getter for RxNorm."""

    bioregistry_id = "rxnorm"
    name = "RxNorm"
    homepage_fmt = "https://download.nlm.nih.gov/umls/kss/rxnorm/RxNorm_full_{version}.zip"
    version_type = VersionType.date

    def get(self) -> datetime:
        """Get the latest RxNorm version number."""
        soup = get_soup(URL)
        raw_version = soup.find("th", {"class": "current"}).contents[2].strip()
        raw_fmt = "%B %d, %Y"
        return datetime.strptime(raw_version, raw_fmt)

    @staticmethod
    def homepage_version_transform(version: str) -> str:
        """Transform date to match RxNorm download URL."""
        return datetime.strftime(datetime.strptime(version, "%Y-%m-%d"), "%m%d%Y")


if __name__ == "__main__":
    RxNormGetter.print()



# -*- coding: utf-8 -*-

"""A getter for StringDB."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "StringDBGetter",
]


class StringDBGetter(Getter):
    """A getter for StringDB."""

    name = "StringDB"
    date_fmt = "%B %d, %Y"
    version_type = VersionType.other

    def get(self):
        """Get the latest StringDB version number."""
        soup = get_soup("https://string-db.org/cgi/access")
        table = soup.find(**{"class": "footer_access_archive_table"})
        rows = table.find_all(**{"class": "row"})
        version, date, link, _summary = [row.text for row in rows[1].find_all(**{"class": "cell"})]
        date = date[len("current: since ") :]
        return dict(version=version, date=date)


if __name__ == "__main__":
    StringDBGetter().print()



# -*- coding: utf-8 -*-

"""A getter for DrugCentral."""

from contextlib import closing

from bioversions.utils import Getter, VersionType

__all__ = [
    "DrugCentralGetter",
]

HOST = "unmtid-dbs.net"
PORT = 5433
USER = "drugman"
PASSWORD = "dosage"
DBNAME = "drugcentral"
PARAMS = dict(dbname=DBNAME, user=USER, password=PASSWORD, host=HOST, port=PORT)


class DrugCentralGetter(Getter):
    """A getter for DrugCentral."""

    bioregistry_id = "drugcentral"
    name = "DrugCentral"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.date

    def get(self):
        """Get the latest DrugCentral version number."""
        import psycopg2

        with closing(psycopg2.connect(**PARAMS)) as conn:
            with closing(conn.cursor()) as cur:
                cur.execute("SELECT version, dtime FROM public.dbversion")
                version, dtime = cur.fetchone()

        # TODO update return format to allow datetime type
        return {
            "version": str(version),
            "date": dtime.strftime(self.date_fmt),
        }


if __name__ == "__main__":
    DrugCentralGetter.print()



# -*- coding: utf-8 -*-

"""A getter for Rfam."""

import ftplib

from bioversions.utils import Getter, VersionType, _is_version

__all__ = [
    "PfamGetter",
]


class PfamGetter(Getter):
    """A getter for Pfam."""

    bioregistry_id = "pfam"
    name = "Pfam"
    homepage_fmt = "ftp://ftp.ebi.ac.uk/pub/databases/Pfam/releases/Pfam{version}/"
    version_type = VersionType.semver_minor

    def get(self):
        """Get the latest Pfam version number."""
        with ftplib.FTP("ftp.ebi.ac.uk") as ftp:
            ftp.login()
            ftp.cwd("pub/databases/Pfam/releases/")
            names = [name[len("Pfam") :] for name in ftp.nlst() if name.startswith("Pfam")]
            names = sorted(
                [
                    tuple(int(part) for part in name.split("."))
                    for name in names
                    if _is_version(name)
                ]
            )
        return ".".join(map(str, names[-1]))


if __name__ == "__main__":
    PfamGetter.print()
